"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAsyncExecutor = exports.execCallbacksAsyncInParallel = exports.execCallbacksAsync = exports.execCallbacks = exports.matchCallbacks = void 0;
function filterByFilters(cb, node) {
    if (typeof cb.filters === 'undefined' || (Array.isArray(cb.filters) && cb.filters.length === 0))
        return true;
    return Array.isArray(cb.filters)
        ? cb.filters.every(f => f(node))
        : cb.filters(node);
}
function filterByNodeType(cb, node) {
    if (!cb.nodeTypeFilters || (Array.isArray(cb.nodeTypeFilters) && cb.nodeTypeFilters.length === 0))
        return true;
    return Array.isArray(cb.nodeTypeFilters)
        ? cb.nodeTypeFilters.indexOf(node.nodeType) !== -1
        : cb.nodeTypeFilters === node.nodeType;
}
function filterByKey(cb, node) {
    if (typeof cb.keyFilters === 'undefined' || (Array.isArray(cb.keyFilters) && cb.keyFilters.length === 0))
        return true;
    if (typeof node.key !== 'string')
        return false;
    return Array.isArray(cb.keyFilters)
        ? cb.keyFilters.indexOf(node.key) !== -1
        : cb.keyFilters === node.key;
}
function matchCallbacks(node, position, ctx) {
    if (!ctx.config.runCallbacks)
        return [];
    if (node.isRoot && !ctx.config.rootObjectCallbacks)
        return [];
    let callbacks = ctx.callbacksByPosition[position];
    return (callbacks || [])
        .map(cb => cb)
        .filter(cb => filterByFilters(cb, node))
        .filter(cb => filterByNodeType(cb, node))
        .filter(cb => filterByKey(cb, node));
}
exports.matchCallbacks = matchCallbacks;
function execCallbacks(callbacks, node) {
    for (let cb of callbacks) {
        cb.callback(node);
        node.executedCallbacks.push(cb);
    }
}
exports.execCallbacks = execCallbacks;
function execCallbacksAsync(callbacks, node) {
    return __awaiter(this, void 0, void 0, function* () {
        for (let cb of callbacks) {
            yield cb.callback(node);
            node.executedCallbacks.push(cb);
        }
    });
}
exports.execCallbacksAsync = execCallbacksAsync;
const execCallbacksAsyncInParallel = (callbacks, node) => __awaiter(void 0, void 0, void 0, function* () {
    yield Promise.all(callbacks.map(cb => Promise.resolve(cb.callback(node))
        .then(() => {
        node.executedCallbacks.push(cb);
    })));
});
exports.execCallbacksAsyncInParallel = execCallbacksAsyncInParallel;
function getAsyncExecutor(config) {
    return config.parallelizeAsyncCallbacks
        ? exports.execCallbacksAsyncInParallel
        : execCallbacksAsync;
}
exports.getAsyncExecutor = getAsyncExecutor;
