"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildContext = exports.buildDefaultContext = exports.buildDefaultConfig = exports.defaultPathFormatter = void 0;
const helpers_1 = require("./helpers");
const defaultPathFormatter = ({ key, isArrayMember }) => isArrayMember ? `[${key}]` : `["${key}"]`;
exports.defaultPathFormatter = defaultPathFormatter;
function buildDefaultConfig() {
    return {
        traversalMode: 'depth',
        rootObjectCallbacks: true,
        runCallbacks: true,
        graphMode: 'finiteTree',
        parallelizeAsyncCallbacks: false,
        callbacks: []
    };
}
exports.buildDefaultConfig = buildDefaultConfig;
function buildDefaultContext(config) {
    return {
        config: Object.assign(Object.assign({}, buildDefaultConfig()), config),
        seenObjects: new Set(),
        callbacksByPosition: {
            "preWalk": [],
            "postWalk": []
        },
    };
}
exports.buildDefaultContext = buildDefaultContext;
function buildContext(config) {
    const ctx = buildDefaultContext(config);
    ctx.config.callbacks.forEach((cb) => {
        const callback = Object.assign(Object.assign({}, cb), { executionOrder: typeof cb.executionOrder == 'undefined' ? 0 : cb.executionOrder, positionFilter: cb.positionFilter || "preWalk" });
        if (callback.positionFilter === "both") {
            ctx.callbacksByPosition["preWalk"].push(callback);
            ctx.callbacksByPosition["postWalk"].push(callback);
        }
        else if (typeof callback.positionFilter !== 'undefined') {
            ctx.callbacksByPosition[callback.positionFilter].push(callback);
        }
    });
    for (const key in ctx.callbacksByPosition)
        ctx.callbacksByPosition[key] = ctx.callbacksByPosition[key].sort(helpers_1.executionOrderSort);
    return ctx;
}
exports.buildContext = buildContext;
