"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WalkNode = void 0;
const defaults_1 = require("./defaults");
const getNormalizedType = (val) => Array.isArray(val)
    ? 'array'
    : typeof val === 'object'
        ? 'object'
        : 'value';
class WalkNode {
    constructor(val, isRoot = false, isArrayMember = false, nodeType = 'value', rawType = 'undefined', executedCallbacks = [], key, parent) {
        this.val = val;
        this.isRoot = isRoot;
        this.isArrayMember = isArrayMember;
        this.nodeType = nodeType;
        this.rawType = rawType;
        this.executedCallbacks = executedCallbacks;
        this.key = key;
        this.parent = parent;
        this.id = WalkNode._idx++;
    }
    static fromRoot(obj) {
        return new WalkNode(obj, true, false, 'object', typeof obj, [], undefined, undefined);
    }
    static fromObjectKey(parent, key) {
        return new WalkNode(parent.val[key], false, false, getNormalizedType(parent.val[key]), typeof parent.val[key], [], key, parent);
    }
    static fromArrayIndex(parent, index) {
        return new WalkNode(parent.val[index], false, true, getNormalizedType(parent.val[index]), typeof parent.val[index], [], index, parent);
    }
    getPath(pathFormat) {
        if (this.isRoot)
            return "";
        pathFormat = pathFormat || defaults_1.defaultPathFormatter;
        return this.parent.getPath(pathFormat) + pathFormat(this);
    }
    get children() {
        var _a;
        if (typeof this._children === 'undefined')
            this._children = [...this.getChildren()];
        return (_a = this._children) !== null && _a !== void 0 ? _a : (this._children = [...this.getChildren()]);
    }
    *getChildren() {
        if (this.nodeType === 'array')
            for (let i = 0; i < this.val.length; i++)
                yield WalkNode.fromArrayIndex(this, i);
        if (this.nodeType === 'object')
            for (let key of Object.keys(this.val))
                yield WalkNode.fromObjectKey(this, key);
    }
    get siblings() {
        var _a, _b;
        return (_b = (_a = this.parent) === null || _a === void 0 ? void 0 : _a.children.filter((c) => c.id !== this.id)) !== null && _b !== void 0 ? _b : [];
    }
}
exports.WalkNode = WalkNode;
WalkNode._idx = 0;
