"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compare = exports.forceEvalAsyncGenerator = exports.forceEvalGenerator = exports.Break = exports.deepCopy = exports.applyAsync = exports.apply = exports.flatten = void 0;
const helpers_1 = require("./helpers");
const walk_1 = require("./walk");
function flatten(obj, key, onlyUnique) {
    //return array of values that match the key
    const arr = [];
    walk_1.walk(obj, {
        callbacks: [{
                keyFilters: [key],
                callback: function (node) {
                    arr.push(node.val);
                }
            }]
    });
    return onlyUnique ? helpers_1.unique(arr) : arr;
}
exports.flatten = flatten;
function apply(obj, ...callbacks) {
    walk_1.walk(obj, { callbacks: callbacks.map(c => ({ callback: c })) });
}
exports.apply = apply;
function applyAsync(obj, ...callbacks) {
    return __awaiter(this, void 0, void 0, function* () {
        yield walk_1.walkAsync(obj, { callbacks: callbacks.map(c => ({ callback: c })) });
    });
}
exports.applyAsync = applyAsync;
function deepCopy(obj) {
    const newObj = {};
    const uuid = 'WALK:DEEP-COPY:DELIMITER';
    const format = ({ key }) => uuid + key;
    walk_1.walk(obj, {
        rootObjectCallbacks: false,
        callbacks: [{
                positionFilter: 'preWalk',
                callback: function (node) {
                    switch (node.nodeType) {
                        case 'array':
                            helpers_1.updateObjectViaPathString(newObj, [], node.getPath(format), uuid);
                            break;
                        case 'object':
                            helpers_1.updateObjectViaPathString(newObj, {}, node.getPath(format), uuid);
                            break;
                        case 'value':
                            helpers_1.updateObjectViaPathString(newObj, node.val, node.getPath(format), uuid);
                            break;
                    }
                }
            }]
    });
    return newObj;
}
exports.deepCopy = deepCopy;
class Break extends Error {
    constructor(message = '') {
        super(message);
        this.name = "Break";
    }
}
exports.Break = Break;
function forceEvalGenerator(gen) {
    for (const _ of gen) { }
}
exports.forceEvalGenerator = forceEvalGenerator;
function forceEvalAsyncGenerator(gen) {
    var gen_1, gen_1_1;
    var e_1, _a;
    return __awaiter(this, void 0, void 0, function* () {
        try {
            for (gen_1 = __asyncValues(gen); gen_1_1 = yield gen_1.next(), !gen_1_1.done;) {
                const _ = gen_1_1.value;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (gen_1_1 && !gen_1_1.done && (_a = gen_1.return)) yield _a.call(gen_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
}
exports.forceEvalAsyncGenerator = forceEvalAsyncGenerator;
const defaultFormatter = ({ key, isArrayMember: isArr }) => isArr ? `[${key}]` : `.${key}`;
function compare(a, b, leavesOnly = false, formatter = defaultFormatter) {
    const aNodes = {};
    const bNodes = {};
    apply(a, n => aNodes[n.getPath(formatter)] = n);
    apply(b, n => bNodes[n.getPath(formatter)] = n);
    return [...new Set([
            ...Object.keys(aNodes),
            ...Object.keys(bNodes)
        ])]
        .filter(key => !leavesOnly || (aNodes[key] || bNodes[key]).nodeType === 'value')
        .map(key => {
        const aNode = aNodes[key];
        const bNode = bNodes[key];
        const removed = aNode && !bNode;
        const added = bNode && !aNode;
        const changed = aNode && bNode && !Object.is(aNode.val, bNode.val);
        let delta = {
            path: key,
            hasDifference: removed || added || changed
        };
        if (added) {
            delta.difference = 'added';
            delta.b = bNode === null || bNode === void 0 ? void 0 : bNode.val;
        }
        else if (removed) {
            delta.difference = 'removed';
            delta.a = aNode === null || aNode === void 0 ? void 0 : aNode.val;
        }
        else if (changed) {
            delta.difference = {
                before: aNode === null || aNode === void 0 ? void 0 : aNode.val,
                after: bNode === null || bNode === void 0 ? void 0 : bNode.val
            };
            delta.a = aNode === null || aNode === void 0 ? void 0 : aNode.val;
            delta.b = bNode === null || bNode === void 0 ? void 0 : bNode.val;
        }
        return delta;
    });
}
exports.compare = compare;
