"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.walkAsync = exports.walk = exports.walkAsyncStep = exports.walkStep = void 0;
const defaults_1 = require("./defaults");
const helpers_1 = require("./helpers");
const utils_1 = require("./utils");
const node_1 = require("./node");
const callback_1 = require("./callback");
class Walker {
    constructor(ctx) {
        this.ctx = ctx;
    }
    shouldSkipVisitation(node) {
        if (node.nodeType === 'value')
            return false;
        if (!this.ctx.seenObjects.has(node.val))
            this.ctx.seenObjects.add(node.val);
        else if (this.ctx.config.graphMode === 'graph')
            return true;
        else if (this.ctx.config.graphMode === 'finiteTree')
            throw "The object violates the defined structure. Override 'graphMode' in the config to allow parsing different object structures.";
        return false;
    }
    get depthFirst() {
        return this.ctx.config.traversalMode === 'depth';
    }
    getPreWalkCallbacks(node) {
        return callback_1.matchCallbacks(node, 'preWalk', this.ctx);
    }
    getPostWalkCallbacks(node) {
        return callback_1.matchCallbacks(node, 'postWalk', this.ctx);
    }
    *walk(obj) {
        const queue = [node_1.WalkNode.fromRoot(obj)];
        const pusher = this.depthFirst
            ? (nodes) => queue.unshift(...nodes)
            : (nodes) => queue.push(...nodes);
        const stacker = new helpers_1.CallbackStacker(callback_1.execCallbacks);
        try {
            do {
                const node = queue.shift();
                if (this.shouldSkipVisitation(node))
                    continue;
                const children = node.children;
                pusher(children);
                stacker.executeOne(node, this.getPreWalkCallbacks(node));
                yield node;
                const afterCbs = this.getPostWalkCallbacks(node);
                if (this.depthFirst && children.length) {
                    const lastChild = children[children.length - 1];
                    stacker.push(lastChild.id, node, afterCbs);
                }
                else {
                    stacker.executeOne(node, afterCbs);
                    utils_1.forceEvalGenerator(stacker.execute(node.id));
                }
            } while (queue.length > 0);
        }
        catch (err) {
            if (!(err instanceof utils_1.Break))
                throw err;
        }
    }
    walkAsync(obj) {
        return __asyncGenerator(this, arguments, function* walkAsync_1() {
            const queue = [node_1.WalkNode.fromRoot(obj)];
            const pusher = this.depthFirst
                ? (nodes) => queue.unshift(...nodes)
                : (nodes) => queue.push(...nodes);
            const executor = callback_1.getAsyncExecutor(this.ctx.config);
            const stacker = new helpers_1.CallbackStacker(executor);
            try {
                do {
                    const node = queue.shift();
                    if (this.shouldSkipVisitation(node))
                        continue;
                    const children = node.children;
                    pusher(children);
                    yield __await(stacker.executeOne(node, this.getPreWalkCallbacks(node)));
                    yield yield __await(node);
                    const afterCbs = this.getPostWalkCallbacks(node);
                    if (this.depthFirst && children.length) {
                        const lastChild = children[children.length - 1];
                        stacker.push(lastChild.id, node, afterCbs);
                    }
                    else {
                        yield __await(stacker.executeOne(node, afterCbs));
                        for (const promise of stacker.execute(node.id))
                            yield __await(promise);
                    }
                } while (queue.length > 0);
            }
            catch (err) {
                if (!(err instanceof utils_1.Break))
                    throw err;
            }
        });
    }
}
function* walkStep(obj, config) {
    const ctx = defaults_1.buildContext(config);
    const walker = new Walker(ctx);
    yield* walker.walk(obj);
}
exports.walkStep = walkStep;
function walkAsyncStep(obj, config) {
    return __asyncGenerator(this, arguments, function* walkAsyncStep_1() {
        const ctx = defaults_1.buildContext(config);
        const walker = new Walker(ctx);
        yield __await(yield* __asyncDelegator(__asyncValues(walker.walkAsync(obj))));
    });
}
exports.walkAsyncStep = walkAsyncStep;
function walk(obj, config) {
    utils_1.forceEvalGenerator(walkStep(obj, config));
}
exports.walk = walk;
function walkAsync(obj, config) {
    return __awaiter(this, void 0, void 0, function* () {
        yield utils_1.forceEvalAsyncGenerator(walkAsyncStep(obj, config));
    });
}
exports.walkAsync = walkAsync;
